-- 弹框管理
Lua_PopupWinManager = class("Lua_PopupWinManager");
function Lua_PopupWinManager:ctor()
	self:superCall("ctor");
	self:dispose();
end

function Lua_PopupWinManager:dispose()
	self.popupWinDataDic = {}; -- 弹窗控制数据
	self.popupWinNameDic = {}; -- 当前弹窗窗口
end

function Lua_PopupWinManager:checkPopupWin()
	if not DataCenter.loginData.isInGameWorld then
		return false;
	end

	if DataCenter.fightData.isFighting then
		return false;
	end

	if not DataCenter.guideData.bInitServerInfo then
		return false;
	end

	local vo;
	local bPopup = false;
	local guideState = DataCenter.guideData.isGuiding;
	for popupType, arr in pairs(self.popupWinDataDic) do
		if self.popupWinNameDic[popupType] == nil then
			local len = arr:getSize();
			for i = 1, len do
				vo = arr[i];
				if vo ~= nil and vo:checkCanPopup(guideState) then -- 引导允许弹框的界面
					bPopup = true;
					break;
				end
			end
			if bPopup == true and vo ~= nil then
				if not Lua_WindowUtil.isWindowShowing(vo.winName) then
					self.popupWinNameDic[popupType] = vo
					Lua_WindowUtil.openWin(vo.winName, Lua_WinParam.ACTION_OPEN, vo.data, true);
				end
				break;
			end
		end
	end
	return bPopup;
end

function Lua_PopupWinManager:checkPopupWinData(winName, act, data)
	if ConfigManager.getInstance().firstConfigLoaded ~= 1 then
		print("未初始化配置checkPopupWinData winName", winName);
        return false;
    end

	if act == Lua_WinParam.ACTION_CLOSE then
		-- self:removeDataByName(winName)
		return false
	end

	local configData = Lua_WinConfigData.getConfigData(winName)
	if configData and configData.popupType > 0 then
		local vo;
		if not DataCenter.loginData.isInGameWorld then -- 未进入游戏场景
			vo = Lua_PopupWinInfoVo.new(winName, act, data, configData)
			self:addData(configData, vo)
			return true
		elseif DataCenter.fightData.isFighting then -- 战斗中
			vo = Lua_PopupWinInfoVo.new(winName, act, data, configData)
			self:addData(configData, vo)
			return true
		elseif not DataCenter.guideData.bInitServerInfo then -- 引导还未初始化
			vo = Lua_PopupWinInfoVo.new(winName, act, data, configData)
			self:addData(configData, vo)
			return true
		elseif DataCenter.guideData.isGuiding then -- 引导中
			if not DataCenter.guideCommonData:checkGuideUsePopupWin(winName) then
				vo = Lua_PopupWinInfoVo.new(winName, act, data, configData)
				self:addData(configData, vo)
				return true
			end
		end

		local curPopupVo = self.popupWinNameDic[configData.popupType];
		if curPopupVo ~= nil then -- 有正在显示的弹框
			if curPopupVo.winName == winName then
				if configData.popupOverlap == true then -- 可叠加
					vo = Lua_PopupWinInfoVo.new(winName, act, data, configData);
					self:addData(configData, vo);
					return true
				end
			else
				if self:checkData(configData) == nil then
					vo = Lua_PopupWinInfoVo.new(winName, act, data, configData);
					self:addData(configData, vo);
					return true
				end
			end
		else
			if act == Lua_WinParam.ACTION_OPEN then
				vo = self:checkData(configData)
				if vo == nil then
					vo = Lua_PopupWinInfoVo.new(winName, act, data, configData)
					self:addData(configData, vo)
				end
				if not vo:checkCanPopup() then
					return true;
				end
				self.popupWinNameDic[configData.popupType] = vo
			end
		end

		-- 检测弹框条件
		if vo ~= nil and not vo:checkCanPopup() then
			return true;
		end
	end

	return false
end

function Lua_PopupWinManager:addData(configData, popupWinVo)
	if self.popupWinDataDic[configData.popupType] == nil then
		self.popupWinDataDic[configData.popupType] = Array();
	end
	self.popupWinDataDic[configData.popupType]:add(popupWinVo);
	self.popupWinDataDic[configData.popupType]:sort("sort", false);
end

function Lua_PopupWinManager:removeData(configData)
	if configData == nil or configData.popupType <= 0 then
		return;
	end

	local curPopupVo = self.popupWinNameDic[configData.popupType];
	local arr = self.popupWinDataDic[configData.popupType];
	if arr then
		local len = arr:getSize();
		if curPopupVo == nil or (curPopupVo and curPopupVo.winName ~= configData.winName) then
			while len > 0 do
				if arr[len].winName == configData.winName then
					self.popupWinDataDic[configData.popupType]:remove(len);
				end
				len = len - 1;
			end
		else
			for i = 1, len do
				if arr[i] == curPopupVo then
					self.popupWinDataDic[configData.popupType]:remove(i);
					break
				end
			end
		end
	end
	
	if curPopupVo and curPopupVo.winName == configData.winName then
		self.popupWinNameDic[configData.popupType] = nil
	end
end

function Lua_PopupWinManager:removeDataByName(winName)
	local configData = Lua_WinConfigData.getConfigData(winName)
	self:removeData(configData)
end

function Lua_PopupWinManager:checkData(configData)
	if configData.popupOverlap == true then -- 可叠加
		return nil
	end

	if self.popupWinDataDic[configData.popupType] then
		local arr = self.popupWinDataDic[configData.popupType]
		for i = 1, arr:getSize() do
			if arr[i].winName == configData.winName then
				return arr[i]
			end
		end
	end
	return nil
end

function Lua_PopupWinManager:generatePopupWinsStr()
	local resultStr = "";
	local hasPopupWin = false;
	for popupType, arr in pairs(self.popupWinDataDic) do
		for i = 1, arr:getSize() do
			local vo = arr[i];
			if vo ~= nil then
				if hasPopupWin then
					resultStr = resultStr .. "、" .. vo.winName;
				else
					hasPopupWin = true;
					resultStr = resultStr .. vo.winName;
				end
			end
		end
	end

	if hasPopupWin then
		return resultStr;
	end
	return "暂无";
end





















-- 弹窗界面数据
Lua_PopupWinInfoVo = class("Lua_PopupWinInfoVo");
function Lua_PopupWinInfoVo:ctor(winName, act, data, configData)
	self:superCall("ctor");
	self.winName = winName;
	self.data = data;
	self.act = act == nil and Lua_WinParam.ACTION_AUTO or act;
	self.configData = configData;
end

function Lua_PopupWinInfoVo.Get:sort()
	return self.configData.popupSort;
end

-- 检测当前该弹框是否可以弹，已排除战斗中情况
function Lua_PopupWinInfoVo:checkCanPopup(guideState)
	if self.configData == nil then
		-- print("checkCanPopup self.configData == nil", self.winName)
		return false;
	end

	-- 带有跳转的窗口在引导中是否可弹出
	if guideState then
		local popupGuideIds = self.configData.popupGuideIds;
		if popupGuideIds ~= nil then
			if popupGuideIds[1] == -1 then
				return false;
			end
			local canPopup;
			local guideId = DataCenter.guideData.curGuideId;
			for i = 1, #popupGuideIds do
				if popupGuideIds[i] == guideId then
					canPopup = true;
					break;
				end
			end

			if not canPopup then
				return false;
			end
		end
	end

	-- 布阵时不允许弹框
	local unPopupLineup = self.configData.unPopupLineup;
	if unPopupLineup ~= nil and unPopupLineup == 1 then
		if DataCenter.heroLineupData.isLineuping then -- 布阵中
			-- print("checkCanPopup 布阵中", self.winName)
			return false;
		end
	end

	local curSceneId = DataCenter.sceneData.curSceneId;
	if curSceneId > 0 then
		-- 允许弹框场景
		local isEmpty = true;
		local allowPopup = false;
		local allowScenes = self.configData.allowScenes;
		if allowScenes ~= nil and #allowScenes > 0 then
			isEmpty = false;
			for i = 1, #allowScenes do
				if allowScenes[i] == curSceneId then
					allowPopup = true;
					break;
				end
			end

			if not allowPopup then
				-- print("checkCanPopup 不在允许弹框场景列表", self.winName)
				return false;
			end
		end

		-- 不允许弹框场景
		if isEmpty or not allowPopup then
			local expectScenes = self.configData.expectScenes;
			if expectScenes ~= nil and #expectScenes > 0 then
				for i = 1, #expectScenes do
					if expectScenes[i] == curSceneId then
						-- print("checkCanPopup 在不允许弹框场景", self.winName)
						return false;
					end
				end
			end
		end
	end

	local unPopupWin = self.configData.unPopupWin;
	if unPopupWin ~= nil then
		for i = 1, #unPopupWin do
			if Lua_WindowUtil.isWindowShowing(unPopupWin[i]) then
				return false;
			end
		end
	end

	-- 引导中不允许弹的
	if guideState and not DataCenter.guideCommonData:checkGuideUsePopupWin(self.winName) then
		-- print("checkCanPopup 引导中不允许弹的", self.winName)
		return false;
	end

	return true;
end